#include <thread>
#include <iostream>
#include <mutex>

class atomic_counter {
public:
    void increment() {
        std::unique_lock<std::mutex> l(m_mutex);
        m_counter++;
    }
    
    void decrement() {
        std::unique_lock<std::mutex> l(m_mutex);
        m_counter--;
    }
    
    int value() {
        std::unique_lock<std::mutex> l(m_mutex);
        return m_counter;
    }
    
private:
    int m_counter = 0;
    std::mutex m_mutex;
};
 
int main() {
    atomic_counter counter;
    
    auto thread_func = [&counter]() {
        for (int i = 0; i < 10'000'000; ++i) {
            counter.increment();
            counter.decrement();
        }
    };
    
    std::thread t1(thread_func);
    std::thread t2(thread_func);
    t1.join();
    t2.join();
    
    std::cout << counter.value() << '\n';
}
